package main

import (
	"encoding/base64"
	"fmt"
	"strings"

	"github.com/valyala/fasthttp"
)

/**
 *	BasicAuth 鉴权
 *
 *	@param ctx				请求上下文
 *	@param channelkey		极光渠道key上
 *	@param masterSecret		极光渠道masterSecret上
 *
 *	@return ok				取鉴权是否成功
 */
func BasicAuth(ctx *fasthttp.RequestCtx, channelkey, masterSecret string) (ok bool) {
	auth := ctx.Request.Header.Peek("Authorization")
	if auth == nil {
		return
	}
	username, password := "", ""
	username, password, ok = parseBasicAuth(string(auth))
	if !ok {
		fmt.Println("parseBasicAuth token fail !")
		return false
	}
	if username != channelkey {
		fmt.Println("channelkey mismatch !  channelkey:", channelkey, "username:", username)
		return false
	}
	if password != masterSecret {
		fmt.Println("masterSecret mismatch !  masterSecret:", masterSecret, "password:", password)
		return false
	}
	return true

}

/**
 *	parseBasicAuth 解析鉴权参数
 *
 *	@param auth				鉴权token
 *
 *	@return username		鉴权用户名
 *	@return password		鉴权用户密码
 *	@return ok				解析鉴权参数是否成功
 */
func parseBasicAuth(auth string) (username, password string, ok bool) {
	const prefix = "Basic "
	if !strings.HasPrefix(auth, prefix) {
		return
	}
	c, err := base64.StdEncoding.DecodeString(auth[len(prefix):])
	if err != nil {
		return
	}
	cs := string(c)
	s := strings.IndexByte(cs, ':')
	if s < 0 {
		return
	}
	return cs[:s], cs[s+1:], true
}
